#include <qpe/mimetype.h>
#include <qpe/applnk.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qiconset.h>
#include <qpe/resource.h>
#include <qmessagebox.h>
#include <qfile.h>
#include <qtextcodec.h>
#include <qtextstream.h>
#include <qsize.h>
#include <qpe/config.h>
#include <qpe/process.h>

#include "../shared/MiscLib.h"
#include "FLib.h"
#include "PreviewBox.h"


PreviewBox::PreviewBox( QWidget* parent,  const char* name, WFlags fl ) : QVBox( parent, name, fl )
{
	sizeX = 320;
	sizeY = 100;
	smooth = false;
	loaded = false;
	textView = new QTextView(this);
	textView->setVScrollBarMode(QScrollView::AlwaysOff);
	textView->setHScrollBarMode(QScrollView::AlwaysOff);
	imageView = new QLabel(this);
	imageView->hide();
	setFocusPolicy(QWidget::NoFocus);

}

PreviewBox::~PreviewBox()
{
}

void PreviewBox::loadCfg()
{
	set.clear();
	Config cf = Config("FileLaunch");
	cf.setGroup("previewset");
	for (int i = 0 ; cf.hasKey("Set_" + FLib::fixNum(i,3)) ; ++ i) {
		const QStringList item = cf.readListEntry("Set_" + FLib::fixNum(i,3),',');
		set[item[0].lower()] = item[1];
	}
}

void PreviewBox::setFontSize(int i)
{
	textView->setFont(QFont("lcfont",i));
}

void PreviewBox::setFile(const QFileInfo& f)
{
	viewFileInfo = f;
	if (! isVisible() ) {
		loaded = false;
		return;
	}
	loaded = true;
	viewFile();
}
void PreviewBox::setSmooth(bool smoothResize)
{
	smooth = smoothResize;
	if (imageView->isVisible()) {
		viewFile();
	}
}

void PreviewBox::viewFile()
{
	QFileInfo f = viewFileInfo;
	QMap<QString,QString>::ConstIterator it = set.find(f.extension(false).lower());
	if (it != set.end()) {
		textView->show();
		imageView->hide();
		Process proc(QStringList() << it.data() << f.filePath() );
		QString out;
		if (proc.exec("",out)) {
			textView->setText(out);
		} else {
			textView->setText("setting error");
		}
		return;
	}
	codename = "";
	int msize = PreviewReadSize;

	int ft = FLib::fileType(f);
	switch (ft) {
	case FLib::Directory:
		imageView->hide();
		textView->setText(tr("Directory"));
		return;
	case FLib::ImageFile:
		textView->hide();
		{
			QPixmap pix;
			if (smooth ) {
				QImage img(f.filePath());
				if (img.height() > height() || img.width() > width() ) {
					float ih = (float)img.height();
					float iw = (float)img.width();
					float hr = (float)img.height() / height();
					float wr = (float)img.width() / width();
					int ww;
					int hh;
					if (hr > wr ) {
						hh = (int)(ih / hr);
						ww = (int)(iw / hr);
					} else {
						hh = (int)(ih / wr);
						ww = (int)(iw / wr);
					}
					img =img.smoothScale(ww , hh );
				}
				pix.convertFromImage( img , QPixmap::AutoColor | QPixmap::ThresholdDither );
			} else {
				pix.load(f.filePath());
				if (pix.height() > height() || pix.width() > width() ) {
					pix.resize(width(),height());
				}
			}
			imageView->setPixmap(pix);
			imageView->show();
		}
		return;
	case FLib::HtmlFile:
		msize = HtmlReadSize;
	case FLib::TextFile:
		{
			textView->show();
			QFile fi(f.filePath());
			if (fi.open(IO_ReadOnly)) {
				imageView->hide();
				int size = (int)f.size();
				if (size > msize) {
					size = msize;
				}
				QString txt;
				if ( size == 0 ) {
					QTextStream t( &fi );
					txt = t.read();
				} else {
					QByteArray buf(size);
					int rsize = fi.readBlock(buf.data(),size);
					if (checkFile((const char *)buf,rsize)) {
						return;
					}

					codename = MiscLib::setCode((const char *)buf,rsize);
					QTextCodec *codec = QTextCodec::codecForName(codename);
					txt = codec->toUnicode(buf,rsize);
				}
				fi.close();
				if (txt.isEmpty()) {
					textView->setText(tr("This file is empty."));
				} else {
					textView->setText(txt);
				}
			}
		}
		return;
	default:
		textView->show();
		imageView->hide();
		textView->setText(FLib::fileTypeName(ft));
		return;
	}
}

void PreviewBox::resizeEvent(QResizeEvent *e)
{
	if (e) {
		if (imageView->isVisible() ) {
			viewFile();
		}
		QVBox::resizeEvent(e);
	}
}

void PreviewBox::showEvent(QShowEvent *e)
{
	if (e) {
		if (! loaded ) {
			viewFile();
		}
		QVBox::showEvent(e);
	}
}

void PreviewBox::hideEvent(QHideEvent *e)
{
	if (e) {
		QVBox::hideEvent(e);
	}
}

bool PreviewBox::checkFile(const char *buf,int fsize)
{
	if (fsize >= 7 && strncmp(buf,binfile,7) == 0 ) {
		textView->setText("binary files");
		return true;
	} else if (fsize >=3 && strncmp(buf,gzipfile,3) == 0) {
		textView->setText("gzip files");
		return true;
	} else if (fsize >=7 && strncmp(buf+2,lzhfile+2,5) == 0) {
		textView->setText("lzh files");
		return true;
	}
	return false;
}

QString PreviewBox::prevFileCode()
{
	return codename;
}


QSize PreviewBox::sizeHint() const
{
	QSize s(sizeX,sizeY);
	return s;
}

void PreviewBox::setSize(int x,int y)
{
	sizeX = x;
	sizeY = y;
}

const char PreviewBox::binfile[] = {0x7f,0x45,0x4c,0x46,0x01,0x01,0x01};
const char PreviewBox::gzipfile[] = {0x1f,0x8b,0x08};
const char PreviewBox::lzhfile[] = {0x00,0x00,0x2d,0x6c,0x68,0x35,0x2d};

